@@ -1,10 +1,10 @@
 use anyhow::Result;
 use deqp_runner::gtest_command::GTestCommand;
 use deqp_runner::mock_gtest::MockGTest;
-use deqp_runner::TestConfiguration;
-use deqp_runner::{parallel_test, process_results, CommandLineRunOptions, TestCommand};
+use deqp_runner::{
+    parallel_test, process_results, CommandLineRunOptions, TestCommand, TestConfiguration,
+};
 use std::path::PathBuf;
-use std::time::Duration;
 use structopt::StructOpt;
 
 #[derive(Debug, StructOpt)]
@@ -66,23 +66,15 @@ fn main() -> Result<()> {
 
             let gtest = GTestCommand {
                 bin: run.gtest,
-                config: TestConfiguration {
-                    output_dir: run.common.output_dir.clone(),
-                    skips: run.common.skips_regex()?,
-                    flakes: run.common.flakes_regex()?,
-                    baseline: run.common.baseline()?,
-                    timeout: Duration::from_secs_f32(run.common.timeout),
-                    env: run.common.env.into_iter().collect(),
-                    save_xfail_logs: run.common.save_xfail_logs,
-                },
+                config: TestConfiguration::from_cli(&run.common)?,
                 args: run.gtest_args,
             };
 
             let tests = gtest
                 .list_tests()?
                 .into_iter()
-                .skip(run.common.fraction_start - 1)
-                .step_by(run.common.fraction)
+                .skip(run.common.sub_config.fraction_start - 1)
+                .step_by(run.common.sub_config.fraction)
                 .filter(|x| include_filter.is_match(x.name()))
                 .collect();
 
