@@ -4,6 +4,7 @@ use crate::{TestCase, TestCommand, TestConfiguration};
 use anyhow::{Context, Result};
 use log::*;
 use regex::Regex;
+use std::collections::HashSet;
 use std::fs::File;
 use std::io::prelude::*;
 use std::io::{BufReader, BufWriter};
@@ -18,12 +19,12 @@ pub struct DeqpCommand {
     pub prefix: String,
 }
 
-fn write_caselist_file(filename: &Path, tests: &[TestCase]) -> Result<()> {
+fn write_caselist_file(filename: &Path, tests: &[&TestCase]) -> Result<()> {
     let file = File::create(filename)
         .with_context(|| format!("creating temp caselist file {}", filename.display()))?;
     let mut file = BufWriter::new(file);
 
-    for test in (&tests).iter() {
+    for test in tests.iter() {
         file.write(test.name().as_bytes())
             .context("writing temp caselist")?;
         file.write(b"\n").context("writing temp caselist")?;
@@ -41,6 +42,34 @@ fn add_filename_arg(args: &mut Vec<String>, arg: &str, path: &Path) -> Result<()
     Ok(())
 }
 
+// Extracts the XML string within a given testcase's section in the QPA.
+pub fn qpa_xml_for_testcase<'a>(qpa: &'a str, test: &str) -> Result<&'a str> {
+    let start = format!("#beginTestCaseResult {}\n", test);
+
+    let xml_after_start = qpa
+        .split(&start)
+        .nth(1)
+        .with_context(|| format!("Finding QPA test start delimiter for {}", test))?;
+    let xml_until_end = xml_after_start
+        .split("#endTestCaseResult")
+        .next()
+        .context("Finding QPA test end delimiter")?;
+
+    Ok(xml_until_end)
+}
+
+// Returns the text from inside the XML's <Text>...</Text> nodes.
+pub fn qpa_xml_text(xml: &str) -> Result<String> {
+    let doc = roxmltree::Document::parse(xml).context("Parsing QPA XML")?;
+
+    Ok(doc
+        .descendants()
+        .filter(|n| n.has_tag_name("Text"))
+        .filter_map(|node| node.text())
+        .collect::<Vec<&str>>()
+        .join("\n"))
+}
+
 impl DeqpCommand {
     fn try_extract_qpa<S: AsRef<str>, P: AsRef<Path>>(&self, test: S, qpa_path: P) -> Result<()> {
         let qpa_path = qpa_path.as_ref();
@@ -85,15 +114,15 @@ impl DeqpCommand {
 
         Ok(())
     }
-    pub fn qpa_text_check(&self, testcase: &str, regex: &str, log_name: &str) -> Result<bool> {
-        let regex = Regex::new(regex)
-            .with_context(|| format!("Compiling QPA text check RE '{}'", regex))?;
+
+    // Runs dEQP for a testcase and collects the QPA file to a string.
+    pub fn deqp_test_qpa_output(&self, testcase: &str, filename: &str) -> Result<String> {
         let qpa_path = self
             .config
             .output_dir
             .canonicalize()
             .context("qpa check canonicalize")?
-            .join(format!("{}.qpa", testcase));
+            .join(filename);
 
         let mut args: Vec<String> = Vec::new();
 
@@ -122,7 +151,8 @@ impl DeqpCommand {
 
         if !output.status.success() {
             anyhow::bail!(
-                "Failed to invoke dEQP for renderer check:\nstdout:\n{}\nstderr:\n{}",
+                "Failed to invoke dEQP for {}:\nstdout:\n{}\nstderr:\n{}",
+                testcase,
                 String::from_utf8_lossy(&output.stdout),
                 String::from_utf8_lossy(&output.stderr)
             );
@@ -130,33 +160,96 @@ impl DeqpCommand {
 
         let mut qpa = String::new();
         File::open(qpa_path)
-            .context("opening renderer check QPA")?
+            .context("opening QPA")?
             .read_to_string(&mut qpa)
-            .context("reading renderer check QPA")?;
+            .context("reading QPA")?;
 
-        let renderer = if testcase == "dEQP-VK.info.device" {
-            qpa.lines()
-                .find(|x| x.contains("deviceName: "))
-                .context("finding deviceName in QPA output")?
-                .trim_start_matches("deviceName: ")
-        } else {
-            // XML regexing :(
-            let text_re = Regex::new("<Text>(.*)</Text>").context("text match RE")?;
-            let text_line = qpa
-                .lines()
-                .skip_while(|line| !line.contains("<TestCaseResult"))
-                .nth(1)
-                .context("reading line after TestCaseResult")?;
+        Ok(qpa)
+    }
 
-            let captures = text_re
-                .captures(&text_line)
-                .context("Finding text line after TestCaseResult")?;
-            captures.get(1).unwrap().as_str()
-        };
+    pub fn qpa_vk_device_name_check(&self, regex: &str) -> Result<bool> {
+        let testcase = "dEQP-VK.info.device";
+        let qpa = self.deqp_test_qpa_output(testcase, testcase)?;
+        let xml = qpa_xml_for_testcase(&qpa, testcase)?;
 
-        println!("{}: {}", log_name, &renderer);
+        for line in qpa_xml_text(xml)?.lines() {
+            if line.starts_with("deviceName: ") {
+                println!("{}", line);
 
-        Ok(regex.is_match(renderer))
+                if !regex.is_empty() {
+                    let regex = Regex::new(regex)
+                        .with_context(|| format!("Compiling QPA text check RE '{}'", regex))?;
+                    return Ok(regex.is_match(line));
+                }
+                return Ok(true);
+            }
+        }
+        anyhow::bail!("Failed to find deviceName")
+    }
+
+    pub fn qpa_gl_renderer_version_check(
+        &self,
+        qpa: &str,
+        testcase: &str,
+        regex: &str,
+        log_name: &str,
+    ) -> Result<bool> {
+        let xml = qpa_xml_for_testcase(qpa, testcase)?;
+        let doc = roxmltree::Document::parse(xml)
+            .with_context(|| format!("Parsing QPA XML for {}", testcase))?;
+
+        for text in doc.descendants().filter(|n| n.has_tag_name("Text")) {
+            if let Some(text) = text.text() {
+                println!("{}: {}", log_name, text);
+                if regex.is_empty() {
+                    return Ok(true);
+                } else {
+                    let regex = Regex::new(regex).with_context(|| {
+                        format!("Compiling QPA renderer/version check RE '{}'", regex)
+                    })?;
+                    return Ok(regex.is_match(text));
+                }
+            }
+        }
+        anyhow::bail!("Failed to find {}", log_name)
+    }
+
+    pub fn qpa_extensions_check(
+        &self,
+        qpa: &str,
+        testcase: &str,
+        extensions_check: &str,
+    ) -> Result<bool> {
+        if extensions_check.is_empty() {
+            return Ok(true);
+        }
+
+        let xml = qpa_xml_for_testcase(qpa, testcase)?;
+        let probed_extensions = qpa_xml_text(xml)?
+            .lines()
+            .map(|x| x.trim().to_string())
+            .collect::<HashSet<String>>();
+
+        let expected_extensions = std::fs::read_to_string(extensions_check)
+            .with_context(|| format!("Reading expected exts file {}", extensions_check))?
+            .lines()
+            .map(|x| x.trim().to_string())
+            .filter(|x| !x.is_empty())
+            .collect::<HashSet<String>>();
+
+        if probed_extensions != expected_extensions {
+            error!("Extensions mismatch:");
+
+            for ext in probed_extensions.difference(&expected_extensions) {
+                error!("Unexpected: {}", ext);
+            }
+            for ext in expected_extensions.difference(&probed_extensions) {
+                error!("Missing: {}", ext);
+            }
+            return Ok(false);
+        }
+
+        Ok(true)
     }
 
     fn qpa_to_xml(&self) -> Option<&PathBuf> {
@@ -165,16 +258,16 @@ impl DeqpCommand {
 }
 
 impl TestCommand for DeqpCommand {
-    fn run<S: AsRef<TestCase>, I: IntoIterator<Item = S>>(
+    fn run(
         &self,
         caselist_state: &CaselistState,
-        tests_o: I,
+        tests_o: &[&TestCase],
     ) -> Result<Vec<RunnerResult>> {
         let caselist_path = self
-            .caselist_file_path(&caselist_state, "caselist.txt")
+            .caselist_file_path(caselist_state, "caselist.txt")
             .context("caselist path")?;
         let qpa_path = self
-            .caselist_file_path(&caselist_state, "qpa")
+            .caselist_file_path(caselist_state, "qpa")
             .context("qpa path")?;
         let cache_path = self
             .config
@@ -183,12 +276,12 @@ impl TestCommand for DeqpCommand {
             .context("cache path")?
             .join(format!("t{}.shader_cache", thread_id::get()));
 
-        let mut tests: Vec<TestCase> = Vec::new();
+        let mut tests: Vec<&TestCase> = Vec::new();
         for test in tests_o {
-            tests.push(test.as_ref().clone());
+            tests.push(test);
         }
 
-        write_caselist_file(&caselist_path, &tests).context("writing caselist file")?;
+        write_caselist_file(&caselist_path, tests.as_slice()).context("writing caselist file")?;
 
         let mut args: Vec<String> = Vec::new();
 
@@ -277,7 +370,7 @@ impl TestCommand for DeqpCommand {
         let mut save_log = false;
         let mut results: Vec<RunnerResult> = Vec::new();
         for result in deqp_results {
-            let status = self.translate_result(&result, &caselist_state);
+            let status = self.translate_result(&result, caselist_state);
 
             if status.should_save_logs(self.config.save_xfail_logs) {
                 save_log = true;
@@ -303,7 +396,7 @@ impl TestCommand for DeqpCommand {
             let stdout = caselist_results.stdout;
 
             let log_path = self
-                .caselist_file_path(&caselist_state, "log")
+                .caselist_file_path(caselist_state, "log")
                 .context("log path")?;
 
             let mut file = File::create(log_path).context("opening log file")?;
@@ -442,4 +535,26 @@ mod tests {
         )
         .is_err());
     }
+
+    #[test]
+    fn filter_qpa_xml_success() -> Result<()> {
+        assert_eq!(
+            include_str!("test_data/deqp-gles2-renderer.xml"),
+            qpa_xml_for_testcase(
+                include_str!("test_data/deqp-gles2-info.qpa"),
+                "dEQP-GLES2.info.renderer"
+            )?
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn filter_qpa_xml_fail() -> Result<()> {
+        assert!(qpa_xml_for_testcase(
+            include_str!("test_data/deqp-gles2-info.qpa"),
+            "dEQP-GLES2.info.notatest"
+        )
+        .is_err());
+        Ok(())
+    }
 }
