@@ -65,10 +65,7 @@ impl GTestResults {
     }
 }
 
-pub fn parse_gtest_results<B>(gtest_output: B) -> Result<GTestResults>
-where
-    B: Read,
-{
+pub fn parse_gtest_results(gtest_output: impl Read) -> Result<GTestResults> {
     let gtest_output = BufReader::new(gtest_output);
     let mut current_test: Option<(String, Instant)> = None;
     lazy_static! {
@@ -125,13 +122,10 @@ where
     Ok(results)
 }
 
-pub fn parse_gtest_results_with_timeout<B>(
-    gtest_output: B,
+pub fn parse_gtest_results_with_timeout(
+    gtest_output: impl Read + std::os::unix::io::AsRawFd,
     timeout: Duration,
-) -> Result<GTestResults>
-where
-    B: Read + std::os::unix::io::AsRawFd,
-{
+) -> Result<GTestResults> {
     parse_gtest_results(TimeoutReader::new(gtest_output, timeout))
 }
 
@@ -162,25 +156,21 @@ impl GTestCommand {
 }
 
 impl TestCommand for GTestCommand {
-    fn run<S: AsRef<TestCase>, I: IntoIterator<Item = S>>(
+    fn run(
         &self,
         caselist_state: &CaselistState,
-        tests: I,
+        tests: &[&TestCase],
     ) -> Result<Vec<RunnerResult>> {
-        let mut tests_iter = tests.into_iter();
+        let mut tests_iter = tests.iter();
 
         let mut tests_arg = format!(
             "--gtest_filter={}",
-            tests_iter
-                .next()
-                .context("getting first test")?
-                .as_ref()
-                .name()
+            tests_iter.next().context("getting first test")?.name()
         );
 
         for test in tests_iter {
             tests_arg.push(':');
-            tests_arg.push_str(test.as_ref().name());
+            tests_arg.push_str(test.name());
         }
 
         let mut command = Command::new(&self.bin);
@@ -236,7 +226,7 @@ impl TestCommand for GTestCommand {
         let mut save_log = false;
         let mut results: Vec<RunnerResult> = Vec::new();
         for result in gtest_results {
-            let status = self.translate_result(&result, &caselist_state);
+            let status = self.translate_result(&result, caselist_state);
 
             if status.should_save_logs(self.config.save_xfail_logs) {
                 save_log = true;
@@ -252,7 +242,7 @@ impl TestCommand for GTestCommand {
 
         if save_log {
             let log_path = self
-                .caselist_file_path(&caselist_state, "log")
+                .caselist_file_path(caselist_state, "log")
                 .context("log path")?;
 
             let mut file = File::create(log_path).context("opening log file")?;
