@@ -22,6 +22,7 @@ use rand::seq::SliceRandom;
 use rand::SeedableRng;
 use rayon::prelude::*;
 use regex::RegexSet;
+use serde::Deserialize;
 use std::collections::HashMap;
 use std::fs::File;
 use std::io::prelude::*;
@@ -45,43 +46,117 @@ where
     Ok((s[..pos].parse()?, s[pos + 1..].parse()?))
 }
 
-#[derive(Debug, StructOpt)]
-pub struct CommandLineRunOptions {
-    #[structopt(long = "output", help = "path to output directory")]
-    pub output_dir: PathBuf,
-
+// Cross test-type CLI/toml options
+#[derive(Clone, Debug, Deserialize, StructOpt)]
+pub struct SubRunConfig {
     #[structopt(
         long,
         help = "path to baseline results (such as output/failures.csv from another run)"
     )]
+    #[serde(default)]
     pub baseline: Option<PathBuf>,
 
     #[structopt(
         long,
         help = "path to file of regexes of tests to skip running (for runtime or stability reasons)"
     )]
+    #[serde(default)]
     pub skips: Vec<PathBuf>,
 
+    #[structopt(
+        long,
+        help = "path to file of regexes of tests to assume any failures in those tests are flaky results (but still run them, for long-term status tracking)"
+    )]
+    #[serde(default)]
+    pub flakes: Vec<PathBuf>,
+
     #[structopt(
         short = "t",
         long = "include-tests",
         help = "regexes of tests to include (non-matching tests are skipped)"
     )]
+    #[serde(default)]
     pub include: Vec<String>,
 
-    #[structopt(
-        long,
-        help = "path to file of regexes of tests to assume any failures in those tests are flaky results (but still run them, for long-term status tracking)"
-    )]
-    pub flakes: Vec<PathBuf>,
-
     #[structopt(
         long,
         default_value = "60.0",
         help = "per-test timeout in floating point seconds"
     )]
+    #[serde(default)]
     pub timeout: f32,
 
+    #[structopt(long, default_value = "1", help = "Runs 1 out of every N tests.")]
+    #[serde(default)]
+    pub fraction: usize,
+
+    #[structopt(
+        long,
+        default_value = "1",
+        help = "Skips the first N-1 tests in the test list before applying --fraction (useful for running N/M fraciton of the test list across multiple devices)."
+    )]
+    #[serde(default)]
+    pub fraction_start: usize,
+
+    #[structopt(
+        parse(try_from_str = parse_key_val),
+        long = "env",
+        help = "Environment variables to set when invoking the test process"
+    )]
+    #[serde(with = "tuple_vec_map", default)]
+    pub env: Vec<(String, String)>,
+}
+
+impl SubRunConfig {
+    pub fn apply_suite_top_config(&mut self, top: &SubRunConfig) {
+        if self.fraction == 0 {
+            self.fraction = 1;
+        }
+        if self.fraction_start == 0 {
+            self.fraction_start = 1;
+        }
+
+        for f in &top.skips {
+            self.skips.push(f.clone());
+        }
+
+        for f in &top.flakes {
+            self.flakes.push(f.clone());
+        }
+
+        if let Some(run_baseline) = top.baseline.as_ref() {
+            if self.baseline.is_some() {
+                eprintln!("baseline may only be set on either the command line or per-deqp.");
+                std::process::exit(1);
+            }
+            self.baseline = Some(run_baseline.clone());
+        }
+
+        // Apply global fraction to the suite's internal fraction.
+        self.fraction *= top.fraction;
+        self.fraction_start += top.fraction_start - 1;
+
+        if self.timeout == 0.0 {
+            self.timeout = 60.0;
+        }
+
+        for (var, data) in &top.env {
+            self.env.push((var.to_owned(), data.to_owned()));
+        }
+    }
+}
+
+// Top-level commandline run/suite options.  Except for env, which ought to be
+// in SubRunConfig but I found hard to get matched up between toml and structopt
+// parsing.
+#[derive(Debug, StructOpt)]
+pub struct CommandLineRunOptions {
+    #[structopt(long = "output", help = "path to output directory")]
+    pub output_dir: PathBuf,
+
+    #[structopt(flatten)]
+    pub sub_config: SubRunConfig,
+
     #[structopt(
         short = "j",
         long,
@@ -90,23 +165,6 @@ pub struct CommandLineRunOptions {
     )]
     pub jobs: usize,
 
-    #[structopt(long, default_value = "1", help = "Runs 1 out of every N tests.")]
-    pub fraction: usize,
-
-    #[structopt(
-        long,
-        default_value = "1",
-        help = "Skips the first N-1 tests in the test list before applying --fraction (useful for running N/M fraciton of the test list across multiple devices)."
-    )]
-    pub fraction_start: usize,
-
-    #[structopt(
-        parse(try_from_str = parse_key_val),
-        long = "env",
-        help = "Environment variables to set when invoking the test process"
-    )]
-    pub env: Vec<(String, String)>,
-
     #[structopt(
         long,
         default_value = "25",
@@ -147,11 +205,11 @@ impl CommandLineRunOptions {
                 .unwrap();
         }
 
-        if self.fraction < 1 {
+        if self.sub_config.fraction < 1 {
             eprintln!("--fraction must be >= 1.");
             std::process::exit(1);
         }
-        if self.fraction_start < 1 {
+        if self.sub_config.fraction_start < 1 {
             eprintln!("--fraction_start must be >= 1.");
             std::process::exit(1);
         }
@@ -162,26 +220,27 @@ impl CommandLineRunOptions {
     }
 
     pub fn baseline(&self) -> Result<RunnerResults> {
-        read_baseline(self.baseline.as_ref())
+        read_baseline(self.sub_config.baseline.as_ref())
     }
 
     pub fn skips_regex(&self) -> Result<RegexSet> {
-        parse_regex_set(read_lines(&self.skips)?).context("compiling skips regexes")
+        parse_regex_set(read_lines(&self.sub_config.skips)?).context("compiling skips regexes")
     }
 
     pub fn flakes_regex(&self) -> Result<RegexSet> {
-        parse_regex_set(read_lines(&self.flakes)?).context("compiling flakes regexes")
+        parse_regex_set(read_lines(&self.sub_config.flakes)?).context("compiling flakes regexes")
     }
 
     pub fn includes_regex(&self) -> Result<RegexSet> {
-        if self.include.is_empty() {
+        if self.sub_config.include.is_empty() {
             RegexSet::new(vec![""]).context("compiling all-tests include RE")
         } else {
-            parse_regex_set(&self.include).context("compiling include filters")
+            parse_regex_set(&self.sub_config.include).context("compiling include filters")
         }
     }
 }
 
+// Common runtime configuration of a test suite sub-run.
 pub struct TestConfiguration {
     pub output_dir: PathBuf,
     pub skips: RegexSet,
@@ -192,14 +251,34 @@ pub struct TestConfiguration {
     pub save_xfail_logs: bool,
 }
 
-pub trait TestCommand {
+impl TestConfiguration {
+    pub fn from_cli(run: &CommandLineRunOptions) -> Result<TestConfiguration> {
+        TestConfiguration::from_suite_config(run, &run.sub_config)
+    }
+
+    pub fn from_suite_config(
+        run: &CommandLineRunOptions,
+        sub_config: &SubRunConfig,
+    ) -> Result<TestConfiguration> {
+        Ok(TestConfiguration {
+            output_dir: run.output_dir.to_path_buf(),
+            skips: parse_regex_set(read_lines(&sub_config.skips)?)
+                .context("compiling skips regexes")?,
+            flakes: parse_regex_set(read_lines(&sub_config.flakes)?)
+                .context("compiling flakes regexes")?,
+            baseline: read_baseline(sub_config.baseline.as_ref())?,
+            timeout: Duration::from_secs_f32(sub_config.timeout),
+            env: sub_config.env.iter().cloned().collect(),
+            save_xfail_logs: run.save_xfail_logs,
+        })
+    }
+}
+
+pub trait TestCommand: Send + Sync {
     fn config(&self) -> &TestConfiguration;
 
-    fn run<S: AsRef<TestCase>, I: IntoIterator<Item = S>>(
-        &self,
-        caselist_state: &CaselistState,
-        tests: I,
-    ) -> Result<Vec<RunnerResult>>;
+    fn run(&self, caselist_state: &CaselistState, tests: &[&TestCase])
+        -> Result<Vec<RunnerResult>>;
 
     fn see_more(&self, _name: &str, _caselist_state: &CaselistState) -> String {
         "".to_string()
@@ -217,8 +296,8 @@ pub trait TestCommand {
         &self.config().baseline
     }
 
-    fn baseline_status<S: AsRef<str>>(&self, test: S) -> Option<RunnerStatus> {
-        self.baseline().tests.get(test.as_ref()).map(|x| x.status)
+    fn baseline_status(&self, test: &str) -> Option<RunnerStatus> {
+        self.baseline().tests.get(test).map(|x| x.status)
     }
 
     fn translate_result(
@@ -238,7 +317,7 @@ pub trait TestCommand {
                 "Test {}: {}: {}",
                 &result.name,
                 status,
-                self.see_more(&result.name, &caselist_state)
+                self.see_more(&result.name, caselist_state)
             );
         }
 
@@ -261,7 +340,7 @@ pub trait TestCommand {
         caselist.sort_by(|x, y| x.name().cmp(y.name()));
 
         caselist_state.run_id += 1;
-        let mut results = self.run(&caselist_state, &caselist)?;
+        let mut results = self.run(caselist_state, caselist.as_slice())?;
         // If we made no more progress on the whole caselist,
         // then dEQP doesn't know about some of our tests and they'll report Missing.
         if results.is_empty() {
@@ -274,7 +353,7 @@ pub trait TestCommand {
         // to see if we get the same results, and mark any changing results as flaky tests.
         if results.iter().any(|x| !x.status.is_success()) {
             caselist_state.run_id += 1;
-            let retest_results = self.run(&caselist_state, &caselist)?;
+            let retest_results = self.run(caselist_state, caselist.as_slice())?;
             for pair in results.iter_mut().zip(retest_results.iter()) {
                 if pair.0.status != pair.1.status {
                     pair.0.status = RunnerStatus::Flake;
@@ -285,15 +364,11 @@ pub trait TestCommand {
         Ok(results)
     }
 
-    fn process_caselist<S: AsRef<TestCase>, I: IntoIterator<Item = S>>(
-        &self,
-        tests: I,
-        caselist_id: u32,
-    ) -> Result<Vec<RunnerResult>> {
+    fn process_caselist(&self, tests: &[TestCase], caselist_id: u32) -> Result<Vec<RunnerResult>> {
         let mut caselist_results: Vec<RunnerResult> = Vec::new();
         let mut remaining_tests = Vec::new();
         for test in tests {
-            let test = test.as_ref().clone();
+            let test = test.clone();
             if self.skip_test(format!("{}{}", self.prefix(), &test.name()).as_str()) {
                 caselist_results.push(RunnerResult {
                     test: format!("{}{}", self.prefix(), test.name()),
@@ -363,7 +438,10 @@ pub trait TestCommand {
         mut tests: Vec<TestCase>,
         tests_per_group: usize,
         min_tests_per_group: usize,
-    ) -> Result<Vec<(&Self, Vec<TestCase>)>> {
+    ) -> Result<Vec<(&dyn TestCommand, Vec<TestCase>)>>
+    where
+        Self: Sized,
+    {
         if tests_per_group < 1 {
             bail!("tests_per_group must be >= 1.");
         }
@@ -391,7 +469,7 @@ pub trait TestCommand {
         // Make test groups of tests_per_group() (512) tests, or if
         // min_tests_per_group() is lower than that, then 1/32nd of the
         // remaining tests down to that limit.
-        let mut test_groups: Vec<(&Self, Vec<TestCase>)> = Vec::new();
+        let mut test_groups: Vec<(&dyn TestCommand, Vec<TestCase>)> = Vec::new();
         let mut remaining = tests.len();
         while remaining != 0 {
             let min = usize::min(min_tests_per_group, remaining);
@@ -429,8 +507,8 @@ pub enum TestCase {
 impl TestCase {
     pub fn name(&self) -> &str {
         match self {
-            TestCase::Deqp(name) => &name,
-            TestCase::GTest(name) => &name,
+            TestCase::Deqp(name) => name,
+            TestCase::GTest(name) => name,
             TestCase::Piglit(test) => &test.name,
         }
     }
@@ -492,17 +570,10 @@ fn results_collection<W: Write>(
 
 // Splits the list of tests to groups and parallelize them across all cores, collecting results in
 // a separate thread
-pub fn parallel_test<D, W>(
-    status_output: W,
-    test_groups: Vec<(&D, Vec<TestCase>)>,
-) -> Result<RunnerResults>
-where
-    D: TestCommand,
-    D: Sync,
-    W: Write,
-    W: Sync,
-    W: Send,
-{
+pub fn parallel_test(
+    status_output: impl Write + Sync + Send,
+    test_groups: Vec<(&dyn TestCommand, Vec<TestCase>)>,
+) -> Result<RunnerResults> {
     let test_count = test_groups.iter().map(|x| x.1.len() as u32).sum();
 
     let mut run_results = RunnerResults::new();
@@ -525,7 +596,7 @@ where
             .enumerate()
             .par_bridge()
             .try_for_each_with(sender, |sender, (i, (deqp, tests))| {
-                sender.send(deqp.process_caselist(tests, i as u32))
+                sender.send(deqp.process_caselist(&tests, i as u32))
             })
             .unwrap();
 
@@ -553,10 +624,7 @@ where
     .context("Parsing regex set")
 }
 
-pub fn read_lines<P, I: IntoIterator<Item = P>>(files: I) -> Result<Vec<String>>
-where
-    P: AsRef<Path>,
-{
+pub fn read_lines<I: IntoIterator<Item = impl AsRef<Path>>>(files: I) -> Result<Vec<String>> {
     let mut lines: Vec<String> = Vec::new();
 
     for path in files {
