@@ -19,12 +19,15 @@ pub struct MockDeqp {
     deqp_log_filename: PathBuf,
 
     #[structopt(long)]
+    #[allow(dead_code)]
     deqp_log_flush: bool,
 
     #[structopt(long, default_value = "", help = "Path to the .shader_cache output")]
+    #[allow(dead_code)]
     deqp_shadercache_filename: PathBuf,
 
     #[structopt(long)]
+    #[allow(dead_code)]
     deqp_shadercache_truncate: bool,
 }
 
@@ -38,10 +41,39 @@ pub fn mock_deqp(mock: &MockDeqp) -> Result<()> {
             .collect::<std::result::Result<Vec<String>, std::io::Error>>()
             .context("reading test caselist")?
     } else {
-        vec![mock.deqp_case.clone()]
+        mock.deqp_case.split(',').map(|x| x.to_string()).collect()
     };
 
     let qpa = File::create(&mock.deqp_log_filename).context("Creating QPA file")?;
+    let mut qpa_writer = BufWriter::new(&qpa);
+
+    // Set up our QPA file before starting to write tests.  We use a snapshot of
+    let qpa_header = if tests[0].contains("dEQP-GLES2") {
+        r#"
+#sessionInfo releaseName git-39e5966401d69eba352d71404827230b90d3063b
+#sessionInfo releaseId 0x39e59664
+#sessionInfo targetName "Surfaceless"
+#sessionInfo vendor "Mesa/X.org"
+#sessionInfo renderer "llvmpipe (LLVM 11.0.1, 256 bits)"
+#sessionInfo commandLineParameters "--deqp-log-images=enable --deqp-log-filename=/home/anholt/TestResults.qpa --deqp-surface-type=pbuffer --deqp-surface-width=256 --deqp-surface-height=256 --deqp-gl-config-name=rgba8888d24s8ms0 --deqp-case=dEQP-GLES2.info.version"
+
+#beginSession
+"#
+    } else {
+        r#"
+#sessionInfo releaseName git-39e5966401d69eba352d71404827230b90d3063b
+#sessionInfo releaseId 0x39e59664
+#sessionInfo targetName "Surfaceless"
+#sessionInfo vendorID 0x1002
+#sessionInfo deviceID 0x687f
+#sessionInfo commandLineParameters "--deqp-log-images=enable --deqp-log-filename=/home/anholt/TestResults.qpa --deqp-surface-type=pbuffer --deqp-surface-width=256 --deqp-surface-height=256 --deqp-gl-config-name=rgba8888d24s8ms0 --deqp-case=dEQP-VK.info.device"
+
+#beginSession
+"#
+    };
+    qpa_writer
+        .write(qpa_header.as_bytes())
+        .context("writing QPA header")?;
 
     for test_name in tests {
         // Missing tests won't appear in the output at all.
@@ -50,6 +82,8 @@ pub fn mock_deqp(mock: &MockDeqp) -> Result<()> {
         }
 
         println!("Test case '{}'..", test_name);
+        writeln!(qpa_writer, "#beginTestCaseResult {}", test_name)
+            .context("writing QPA test start")?;
 
         if test_name.contains(".timeout.") {
             // Simulate a testcase that doesn't return in time by infinite
@@ -76,24 +110,41 @@ pub fn mock_deqp(mock: &MockDeqp) -> Result<()> {
             // handles the deqp output unexpectedly ending as a crash.
             break;
         } else if test_name == "dEQP-GLES2.info.renderer" {
-            BufWriter::new(&qpa)
-                .write(include_bytes!("test_data/deqp-gles2-renderer.qpa"))
-                .context("writing QPA")?;
+            qpa_writer
+                .write(include_bytes!("test_data/deqp-gles2-renderer.xml"))
+                .context("writing QPA XML")?;
             println!("  Pass (success case)");
         } else if test_name == "dEQP-GLES2.info.version" {
-            BufWriter::new(&qpa)
-                .write(include_bytes!("test_data/deqp-gles2-version.qpa"))
-                .context("writing QPA")?;
+            qpa_writer
+                .write(include_bytes!("test_data/deqp-gles2-version.xml"))
+                .context("writing QPA XML")?;
+            println!("  Pass (success case)");
+        } else if test_name == "dEQP-GLES2.info.extensions" {
+            qpa_writer
+                .write(include_bytes!("test_data/deqp-gles2-extensions.xml"))
+                .context("writing QPA XML")?;
             println!("  Pass (success case)");
         } else if test_name == "dEQP-VK.info.device" {
-            BufWriter::new(&qpa)
-                .write(include_bytes!("test_data/deqp-vk-info-device.qpa"))
-                .context("writing QPA")?;
+            qpa_writer
+                .write(include_bytes!("test_data/deqp-vk-info-device.xml"))
+                .context("writing QPA XML")?;
             println!("  Pass (success case)");
         } else {
             unimplemented!("unknown mock test name {}", test_name)
         }
+
+        writeln!(qpa_writer, "\n#endTestCaseResult").context("writing QPA test end")?;
     }
 
+    writeln!(
+        qpa_writer,
+        r#"
+#endTestsCasesTime
+
+#endSession
+"#
+    )
+    .context("writing QPA footer")?;
+
     Ok(())
 }
