@@ -64,10 +64,7 @@ impl PartialEq for DeqpTestResult {
     }
 }
 
-pub fn parse_deqp_results<B>(deqp_output: B) -> Result<DeqpCaselistResult>
-where
-    B: Read,
-{
+pub fn parse_deqp_results(deqp_output: impl Read) -> Result<DeqpCaselistResult> {
     let deqp_output = BufReader::new(deqp_output);
     let mut current_test: Option<(String, Instant)> = None;
     lazy_static! {
@@ -101,7 +98,14 @@ where
         let line = stdout.last().unwrap();
 
         if let Some((ref name, time)) = current_test {
-            if let Some(cap) = STATUS_RE.captures(&line) {
+            if let Some(cap) = STATUS_RE.captures(line) {
+                let status = &cap[1];
+
+                /* One of the VK sparse tests emits some info lines about a NotSupported before actually saying NotSupported. */
+                if status == "Info" {
+                    continue;
+                }
+
                 let status = DeqpStatus::from_str(&cap[1])?;
 
                 results.push(DeqpTestResult {
@@ -111,7 +115,7 @@ where
                 });
                 current_test = None;
             }
-        } else if let Some(cap) = TEST_RE.captures(&line) {
+        } else if let Some(cap) = TEST_RE.captures(line) {
             current_test = Some((cap[1].to_string(), Instant::now()));
         }
     }
@@ -126,13 +130,10 @@ where
     Ok(DeqpCaselistResult { results, stdout })
 }
 
-pub fn parse_deqp_results_with_timeout<B>(
-    deqp_output: B,
+pub fn parse_deqp_results_with_timeout(
+    deqp_output: impl Read + std::os::unix::io::AsRawFd,
     timeout: Duration,
-) -> Result<DeqpCaselistResult>
-where
-    B: Read + std::os::unix::io::AsRawFd,
-{
+) -> Result<DeqpCaselistResult> {
     parse_deqp_results(TimeoutReader::new(deqp_output, timeout))
 }
 
@@ -357,4 +358,20 @@ Test case 'dEQP-GLES2.test.t'..
 
         Ok(())
     }
+
+    #[test]
+    fn parse_vk_sparse_info() -> Result<()> {
+        let output = r#"
+Test case 'dEQP-VK.api.buffer_memory_requirements.create_sparse_binding_sparse_residency_sparse_aliased.ext_mem_flags_included.method1.size_req_transfer_usage_bits'..
+  Info (Create buffer with VK_BUFFER_CREATE_SPARSE_BINDING_BIT not supported by device at vktApiBufferMemoryRequirementsTests.cpp:353)
+  Info (Create buffer with VK_BUFFER_CREATE_SPARSE_RESIDENCY_BIT not supported by device at vktApiBufferMemoryRequirementsTests.cpp:359)
+  Info (Create buffer with VK_BUFFER_CREATE_SPARSE_ALIASED_BIT not supported by device at vktApiBufferMemoryRequirementsTests.cpp:365)
+  NotSupported (One or more create buffer flags not supported by device at vktApiBufferMemoryRequirementsTests.cpp:377)"#;
+
+        let caselist_results = parse_deqp_results(&mut output.as_bytes())?;
+        let results = caselist_results.results;
+        assert_eq!(results[0].status, DeqpStatus::NotSupported);
+
+        Ok(())
+    }
 }
