@@ -67,10 +67,7 @@ impl PartialEq for PiglitTestResult {
     }
 }
 
-pub fn parse_piglit_results<B>(piglit_output: B) -> PiglitTestResult
-where
-    B: Read,
-{
+pub fn parse_piglit_results(piglit_output: impl Read) -> PiglitTestResult {
     let piglit_output = BufReader::new(piglit_output);
 
     // We only parse the final result, not any subtests, as we don't have a
@@ -152,10 +149,10 @@ where
     }
 }
 
-pub fn parse_piglit_results_with_timeout<B>(deqp_output: B, timeout: Duration) -> PiglitTestResult
-where
-    B: Read + std::os::unix::io::AsRawFd,
-{
+pub fn parse_piglit_results_with_timeout(
+    deqp_output: impl Read + std::os::unix::io::AsRawFd,
+    timeout: Duration,
+) -> PiglitTestResult {
     parse_piglit_results(TimeoutReader::new(deqp_output, timeout))
 }
 
@@ -295,7 +292,7 @@ pub fn parse_piglit_xml_testlist(
     file_content: &str,
     process_isolation: bool,
 ) -> Result<Vec<crate::TestCase>> {
-    let doc = Document::parse(&file_content).context("reading caselist")?;
+    let doc = Document::parse(file_content).context("reading caselist")?;
 
     let re = Regex::new(r"['\[\]]").unwrap();
     let mut tests = Vec::new();
@@ -304,7 +301,7 @@ pub fn parse_piglit_xml_testlist(
     for test in doc.descendants().filter(|n| n.has_tag_name("Profile")) {
         if let Some(name) = test.text() {
             info!("Found subprofile: {:?}", name);
-            let content = read_profile_file(folder, &name, process_isolation)?;
+            let content = read_profile_file(folder, name, process_isolation)?;
             for t in parse_piglit_xml_testlist(folder, &content, process_isolation)? {
                 tests.push(t);
             }
@@ -369,7 +366,7 @@ pub fn parse_piglit_xml_testlist(
             }
 
             Some(test_type) => {
-                let command = get_option_value(&test, &"command").context("parsing command")?;
+                let command = get_option_value(&test, "command").context("parsing command")?;
 
                 let run_concurrent = match get_option_value(&test, "run_concurrent")
                     .context("parsing concurrent")?
@@ -403,7 +400,7 @@ mod tests {
 
     fn parse_immediate_xml(xml: &str) -> Result<Vec<TestCase>> {
         let dummy_path = PathBuf::from(".");
-        parse_piglit_xml_testlist(&dummy_path, &xml, false)
+        parse_piglit_xml_testlist(&dummy_path, xml, false)
     }
 
     #[test]
@@ -415,7 +412,7 @@ mod tests {
         </Test>"#;
 
         assert_eq!(
-            parse_immediate_xml(&xml).unwrap()[0],
+            parse_immediate_xml(xml).unwrap()[0],
             TestCase::Piglit(crate::PiglitTest {
                 name: "fast_color_clear@fcc-read-after-clear blit rb".to_string(),
                 binary: "fcc-read-after-clear".to_string(),
@@ -438,7 +435,7 @@ mod tests {
         </Test>"#;
 
         assert_eq!(
-            parse_immediate_xml(&xml).unwrap()[0],
+            parse_immediate_xml(xml).unwrap()[0],
             TestCase::Piglit(crate::PiglitTest {
                 name: "asmparsertest@arbfp1.0@cos-03.txt".to_string(),
                 binary: "asmparsertest".to_string(),
@@ -461,7 +458,7 @@ mod tests {
         </Test>"#;
 
         assert_eq!(
-            parse_immediate_xml(&xml).unwrap()[0],
+            parse_immediate_xml(xml).unwrap()[0],
             TestCase::Piglit(crate::PiglitTest {
                 name: "spec@ext_clip_cull_distance@preprocessor@disabled-defined-es.comp"
                     .to_string(),
@@ -485,7 +482,7 @@ mod tests {
         </Test>"#;
 
         assert_eq!(
-            parse_immediate_xml(&xml).unwrap()[0],
+            parse_immediate_xml(xml).unwrap()[0],
             TestCase::Piglit(crate::PiglitTest {
                 name: "glx@glx-query-drawable-glxbaddrawable".to_string(),
                 binary: "glx-query-drawable".to_string(),
@@ -649,7 +646,7 @@ PIGLIT: {\"result\": \"pass\" }";
     #[test]
     fn parse_timeout_no_result() -> Result<()> {
         let output = "Starting piglit test...\n";
-        assert_eq!(timeout_test(&output)?, result(DeqpStatus::Timeout, output),);
+        assert_eq!(timeout_test(output)?, result(DeqpStatus::Timeout, output),);
         Ok(())
     }
 
