@@ -2,20 +2,94 @@ use crate::parse_deqp::{DeqpStatus, DeqpTestResult};
 use crate::parse_piglit::{
     parse_piglit_results_with_timeout, piglit_sanitize_test_name, PiglitTestResult,
 };
-use crate::{runner_results::*, TestConfiguration};
+use crate::parse_piglit::{parse_piglit_xml_testlist, read_profile_file};
+use crate::{parse_regex_set, runner_results::*, SubRunConfig, TestConfiguration};
 use crate::{TestCase, TestCommand};
 use anyhow::{Context, Result};
 use log::*;
+use serde::Deserialize;
 use std::fs::File;
 use std::io::prelude::*;
 use std::io::BufReader;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
+use structopt::StructOpt;
 
 pub struct PiglitCommand {
     pub config: TestConfiguration,
     pub piglit_folder: PathBuf,
+    pub prefix: String,
+}
+
+// Common structure for configuring a piglit run between Run (single run) and deqp-runner Suite (muliple Runs)
+#[derive(Debug, Deserialize, StructOpt)]
+pub struct PiglitRunConfig {
+    #[structopt(long, help = "path to piglit folder")]
+    pub piglit_folder: PathBuf,
+
+    #[structopt(long, help = "piglit profile to run (such as quick_gl)")]
     pub profile: String,
+
+    #[structopt(long = "process-isolation")]
+    #[serde(default)]
+    pub process_isolation: bool,
+}
+
+#[derive(Deserialize)]
+pub struct PiglitTomlConfig {
+    #[serde(flatten)]
+    pub sub_config: SubRunConfig,
+
+    #[serde(flatten)]
+    pub piglit_config: PiglitRunConfig,
+
+    #[serde(default)]
+    pub prefix: String,
+}
+
+impl PiglitTomlConfig {
+    pub fn test_groups<'d>(
+        &self,
+        piglit: &'d PiglitCommand,
+        filters: &[String],
+    ) -> Result<Vec<(&'d dyn TestCommand, Vec<TestCase>)>> {
+        let mut include_filters = Vec::new();
+        if !self.sub_config.include.is_empty() {
+            include_filters.push(
+                parse_regex_set(&self.sub_config.include).context("compiling include filters")?,
+            );
+        }
+        if !filters.is_empty() {
+            include_filters.push(parse_regex_set(filters).context("compiling include filters")?);
+        }
+
+        let test_folder = self.piglit_config.piglit_folder.join("tests");
+        let text = read_profile_file(
+            &test_folder,
+            &self.piglit_config.profile,
+            self.piglit_config.process_isolation,
+        )?;
+        let tests: Vec<TestCase> =
+            parse_piglit_xml_testlist(&test_folder, &text, self.piglit_config.process_isolation)
+                .with_context(|| {
+                    format!("reading piglit profile '{}'", &self.piglit_config.profile)
+                })?
+                .into_iter()
+                .skip(self.sub_config.fraction_start - 1)
+                .step_by(self.sub_config.fraction)
+                .filter(|test| include_filters.iter().all(|x| x.is_match(test.name())))
+                .collect();
+
+        println!(
+            "Running {} piglit tests on {} threads",
+            tests.len(),
+            rayon::current_num_threads()
+        );
+
+        let groups = piglit.split_tests_to_groups(tests, 1, 1)?;
+
+        Ok(groups)
+    }
 }
 
 #[derive(Debug, Eq, PartialEq, Clone)]
@@ -26,16 +100,14 @@ pub struct PiglitTest {
 }
 
 impl TestCommand for PiglitCommand {
-    fn run<S: AsRef<TestCase>, I: IntoIterator<Item = S>>(
+    fn run(
         &self,
         caselist_state: &CaselistState,
-        tests: I,
+        tests: &[&TestCase],
     ) -> Result<Vec<RunnerResult>> {
         let mut bin_path = self.piglit_folder.clone();
         bin_path.push("bin");
 
-        let tests: Vec<TestCase> = tests.into_iter().map(|x| x.as_ref().clone()).collect();
-
         // We only run one piglit command in a test group.  This means that
         // flake detection doesn't have to run irrelevant tests, and makes our
         // log file handling easier.
@@ -126,7 +198,7 @@ impl TestCommand for PiglitCommand {
                 status: piglit_result.status.unwrap_or(DeqpStatus::Crash),
                 duration: piglit_result.duration,
             },
-            &caselist_state,
+            caselist_state,
         );
 
         for subtest in &piglit_result.subtests {
@@ -148,14 +220,16 @@ impl TestCommand for PiglitCommand {
                         status: subtest.status,
                         duration: subtest.duration,
                     },
-                    &caselist_state,
+                    caselist_state,
                 ),
                 duration: subtest.duration,
                 subtest: true,
             });
         }
 
-        if translated_result.should_save_logs(self.config.save_xfail_logs) {
+        if translated_result.should_save_logs(self.config.save_xfail_logs)
+            || test.name.contains("glinfo")
+        {
             let mut file = File::create(log_path).context("opening log file")?;
 
             fn write_output(file: &mut File, name: &str, out: &[String]) -> Result<()> {
