@@ -308,10 +308,7 @@ impl RunnerResults {
         Ok(())
     }
 
-    pub fn write_failures<W>(&self, writer: &mut W) -> Result<()>
-    where
-        W: Write,
-    {
+    pub fn write_failures(&self, writer: &mut impl Write) -> Result<()> {
         let mut sorted: Vec<_> = self.tests.iter().collect();
         sorted.sort_by(|x, y| (*x).0.cmp((*y).0));
 
@@ -324,14 +321,11 @@ impl RunnerResults {
         Ok(())
     }
 
-    pub fn write_junit_failures<W>(
+    pub fn write_junit_failures(
         &self,
-        writer: &mut W,
+        writer: &mut impl Write,
         options: &JunitGeneratorOptions,
-    ) -> Result<()>
-    where
-        W: Write,
-    {
+    ) -> Result<()> {
         use junit_report::*;
         let mut sorted: Vec<_> = self.tests.iter().collect();
         sorted.sort_by(|x, y| (*x).0.cmp((*y).0));
@@ -364,10 +358,7 @@ impl RunnerResults {
             .context("writing XML output")
     }
 
-    pub fn from_csv<R: Read>(r: &mut R) -> Result<RunnerResults>
-    where
-        R: Read,
-    {
+    pub fn from_csv(r: &mut impl Read) -> Result<RunnerResults> {
         lazy_static! {
             static ref CSV_RE: Regex = Regex::new("^([^,]+),([^,]+)").unwrap();
         }
@@ -674,11 +665,11 @@ dEQP-GLES2.info.version,Pass"
         add_result(&mut results, "pass1", RunnerStatus::Pass);
         add_result(&mut results, "pass2", RunnerStatus::Pass);
 
-        assert_eq!(results.is_success(), true);
+        assert!(results.is_success());
 
         add_result(&mut results, "Crash", RunnerStatus::Crash);
 
-        assert_eq!(results.is_success(), false);
+        assert!(!results.is_success());
     }
 }
 
