@@ -1,7 +1,8 @@
 use super::*;
 use ::deqp_runner::{RunnerResults, RunnerStatus};
 use anyhow::{Context, Result};
-use std::io::Cursor;
+use std::fs::File;
+use std::io::{BufReader, Cursor};
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 
@@ -39,6 +40,7 @@ struct DeqpMock {
     pub prefix: String,
     pub renderer_check: String,
     pub version_check: String,
+    pub extensions_check: Option<MaybeTempFile>,
     pub includes: Vec<String>,
 }
 
@@ -66,7 +68,7 @@ impl DeqpMock {
         let child = child.arg(output_dir.path());
 
         let child = child.arg("--timeout");
-        let mut child = child.arg("1");
+        let child = child.arg("1");
 
         for caselist_file in &self.caselists {
             child.arg("--caselist");
@@ -98,7 +100,12 @@ impl DeqpMock {
             child.arg(&self.version_check);
         }
 
-        add_includes(&mut child, &self.includes);
+        if let Some(file) = &self.extensions_check {
+            child.arg("--extensions-check");
+            child.arg(file.as_ref().as_os_str());
+        }
+
+        add_includes(child, &self.includes);
 
         for arg in &self.runner_args {
             child.arg(arg);
@@ -144,28 +151,19 @@ impl DeqpMock {
         self
     }
 
-    pub fn with_baseline<S>(&mut self, data: S) -> &mut DeqpMock
-    where
-        S: AsRef<str>,
-    {
+    pub fn with_baseline(&mut self, data: impl AsRef<str>) -> &mut DeqpMock {
         self.baselines
             .push(tempfile(data).context("writing baseline").unwrap());
         self
     }
 
-    pub fn with_skips<S>(&mut self, data: S) -> &mut DeqpMock
-    where
-        S: AsRef<str>,
-    {
+    pub fn with_skips(&mut self, data: impl AsRef<str>) -> &mut DeqpMock {
         self.skips
             .push(tempfile(data).context("writing skips").unwrap());
         self
     }
 
-    pub fn with_flakes<S>(&mut self, data: S) -> &mut DeqpMock
-    where
-        S: AsRef<str>,
-    {
+    pub fn with_flakes(&mut self, data: impl AsRef<str>) -> &mut DeqpMock {
         self.flakes
             .push(tempfile(data).context("writing flakes").unwrap());
         self
@@ -190,10 +188,16 @@ impl DeqpMock {
         self.renderer_check = arg.to_owned();
         self
     }
+
     pub fn with_version_check(&mut self, arg: &str) -> &mut DeqpMock {
         self.version_check = arg.to_owned();
         self
     }
+
+    pub fn with_extensions_check(&mut self, path: MaybeTempFile) -> &mut DeqpMock {
+        self.extensions_check = Some(path);
+        self
+    }
 }
 
 fn mocked_deqp_runner<S: AsRef<str>>(tests: Vec<S>) -> RunnerResults {
@@ -265,6 +269,9 @@ impl DeqpSuite {
                 if !deqp.version_check.is_empty() {
                     writeln!(toml, r#"version_check = "{}""#, &deqp.version_check)?;
                 }
+                if let Some(path) = &deqp.extensions_check {
+                    writeln!(toml, r#"extensions_check = "{:?}""#, path.as_ref())?;
+                }
 
                 if !deqp.includes.is_empty() {
                     write!(toml, r#"include = ["#,)?;
@@ -302,9 +309,9 @@ impl DeqpSuite {
         let child = child.arg(output_dir.path());
 
         let child = child.arg("--suite");
-        let mut child = child.arg(toml.path());
+        let child = child.arg(toml.path());
 
-        add_includes(&mut child, &self.includes);
+        add_includes(child, &self.includes);
 
         let output = child
             .spawn()
@@ -636,8 +643,7 @@ fn includes() {
 
 #[test]
 fn caselist_subdir() {
-    let mut caselist = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
-    caselist.push("resources/test/caselist-subdir.txt");
+    let caselist = test_resource_path("caselist-subdir.txt");
 
     let results = DeqpMock::new().with_caselist_file(&caselist).run().unwrap();
 
@@ -889,6 +895,77 @@ fn renderer_check_fail() {
     assert_eq!(Some(1), results.status.code());
 }
 
+#[test]
+fn gl_extensions_check() {
+    let exts_path = test_resource_path("deqp-gles2-extensions.txt");
+
+    let mut deqp = DeqpMock::new();
+    deqp.with_cases(vec!["dEQP-GLES2.test.p.1"]);
+    deqp.with_extensions_check(MaybeTempFile::File(exts_path));
+
+    let results = deqp.run().unwrap();
+
+    assert_eq!(Some(0), results.status.code());
+
+    let results = results.results.unwrap();
+    assert_eq!(results.result_counts.pass, 1);
+    assert_eq!(results.result_counts.fail, 0);
+}
+
+#[test]
+fn gl_extensions_check_unexpected() -> Result<()> {
+    // List of exts to check for, with a bunch of exts missing that the actual
+    // impl has.
+    let test_ext_file = lines_tempfile(vec!["GL_EXT_base_instance", "GL_EXT_draw_instanced"])
+        .context("writing list of extensions to test for")?;
+
+    let mut deqp = DeqpMock::new();
+    deqp.with_cases(vec!["dEQP-GLES2.test.p.1"]);
+    deqp.with_extensions_check(MaybeTempFile::Temp(test_ext_file));
+
+    let results = deqp.run().unwrap();
+
+    assert_eq!(Some(1), results.status.code());
+
+    // Check that we mentioned one of the unexpected exts.
+    assert!(results.stderr.contains("Unexpected: GL_EXT_frag_depth"));
+
+    Ok(())
+}
+
+#[test]
+fn gl_extensions_check_missing() -> Result<()> {
+    let mut exts = Vec::new();
+    for ext in BufReader::new(
+        File::open(test_resource_path("deqp-gles2-extensions.txt")).context("opening exts list")?,
+    )
+    .lines()
+    {
+        let ext = ext.context("reading from exts list")?;
+        exts.push(ext.clone());
+    }
+
+    // List of exts to check for, with an extra one
+    let missing_ext = "GL_ARB_ham_sandwich";
+    exts.push(missing_ext.to_owned());
+    let test_ext_file = lines_tempfile(exts).context("writing list of extensions to test for")?;
+
+    let mut deqp = DeqpMock::new();
+    deqp.with_cases(vec!["dEQP-GLES2.test.p.1"]);
+    deqp.with_extensions_check(MaybeTempFile::Temp(test_ext_file));
+
+    let results = deqp.run().unwrap();
+
+    assert_eq!(Some(1), results.status.code());
+
+    // Check that we mentioned the missing ext.
+    assert!(results
+        .stderr
+        .contains(&format!("Missing: {}", missing_ext)));
+
+    Ok(())
+}
+
 #[test]
 fn version_check_fail() {
     let mut deqp = DeqpMock::new();
@@ -908,14 +985,14 @@ fn version_check_fail() {
 }
 
 #[test]
-fn vk_renderer_version_check() {
+fn vk_renderer_check() {
     let mut deqp = DeqpMock::new();
     deqp.with_cases(vec!["dEQP-VK.test.p.1"]);
     deqp.with_renderer_check("AMD RADV");
 
     let results = deqp.run().unwrap();
 
-    assert!(results.stdout.contains("renderer: AMD RADV VEGA10"));
+    assert!(results.stdout.contains("deviceName: AMD RADV VEGA10"));
 
     assert_eq!(Some(0), results.status.code());
 
@@ -932,7 +1009,7 @@ fn vk_renderer_check_fail() {
 
     let results = deqp.run().unwrap();
 
-    assert!(results.stdout.contains("renderer: AMD RADV VEGA10"));
+    assert!(results.stdout.contains("deviceName: AMD RADV VEGA10"));
     assert_eq!(Some(1), results.status.code());
 }
 
